nodes=()
vip=""
ip=$(ifconfig | awk 'NR==2{print $2}')

function success(){
  echo -e "\e[92m"$1"\e[0m"
}
function warn(){
  echo -e "\e[93m"$1"\e[0m"
}
function error(){
  echo -e "\e[91m"$1"\e[0m"
}

while getopts "n:v:" opt; do
  case ${opt} in
    n )
      nodes+=("$OPTARG")  # 将每个 -n 的值加入数组
      ;;
    v )
      vip=$OPTARG
      ;;
    \? )
      warn "Usage: cmd [-n node_ip] [-v vip]"
      exit 1
      ;;
  esac
done

node_length=${#nodes[@]}

if [[ $node_length -lt 2 ]]; then
  error "集群最少需要3个节点"
  exit 1
fi

if [[ -z "$vip" ]]; then
  error "VIP不允许为空"
  exit 1
fi

echo "进入目录$(dirname $0)"


echo "更新数据库配置"

psql -d db_informat2_account_prd -U postgres -h 127.0.0.1 -p 9999 -c "update t_config set value = 'http://$vip/' where id='system.web.url'"
psql -d db_informat2_account_prd -U postgres -h 127.0.0.1 -p 9999 -c "update t_config set value = 'http://$vip:9002' where id='preview.url'"
psql -d db_informat2_account_prd -U postgres -h 127.0.0.1 -p 9999 -c "update t_config set value = 'http://$vip:19002' where id='fileservice.s3.endPoint'"

success "更新数据库配置成功"

cd "$(dirname "$0")"

SSH_OPTIONS="-o StrictHostKeyChecking=no -o UserKnownHostsFile=/dev/null -i ~/.ssh/id_rsa"

redis_nodes=""

for node in "${nodes[@]}"; do
redis_nodes+="        - $node:6381
        - $node:6382
"
done

account_yml_tmpl=$(cat './account.yml.tmpl')
biz_yml_tmpl=$(cat './biz.yml.tmpl')

node_id=0
for node in "${nodes[@]}"; do
account_yml=${account_yml_tmpl//\{vip\}/$vip}
account_yml=${account_yml//\{redis_nodes\}/$redis_nodes}
account_yml=${account_yml//\{node_id\}/$node_id}

biz_yml=${biz_yml_tmpl//\{vip\}/$vip}
biz_yml=${biz_yml//\{redis_nodes\}/$redis_nodes}
biz_yml=${biz_yml//\{node_id\}/$node_id}

echo "更新${node}的配置"

ssh -T $SSH_OPTIONS root@$node << EOF
mv /informat-next/instance/informat-account/application.yml /informat-next/instance/informat-account/application.yml.bak
echo "$account_yml" > /informat-next/instance/informat-account/application.yml
mv /informat-next/instance/informat-biz/application.yml /informat-next/instance/informat-biz/application.yml.bak
echo "$biz_yml" > /informat-next/instance/informat-biz/application.yml

systemctl restart informat-account
sleep 10
systemctl restart informat-biz
EOF

success "更新${node}配置成功"

((node_id++))
done