#!/bin/bash

# 定义设备
nodes=() # 替换为你的设备，例如 /dev/sda
device=""

while getopts "n:d:" opt; do
  case ${opt} in
  n)
    nodes+=("$OPTARG") # 将每个 -n 的值加入数组
    ;;
  d)
    device=$OPTARG
    ;;
  \?)
    warn "Usage: cmd [-n node_ip] [-d device]"
    exit 1
    ;;
  esac
done

function success(){
  echo -e "\e[92m"$1"\e[0m"
}
function warn(){
  echo -e "\e[93m"$1"\e[0m"
}
function error(){
  echo -e "\e[91m"$1"\e[0m"
}

if [ ! -e "$device" ]; then
  error "${device}磁盘不存在"
  exit 1
fi

systemctl stop minio

echo "开启防火墙端口"

firewall-cmd --zone=public --add-port=19000/tcp --permanent
firewall-cmd --zone=public --add-port=19001/tcp --permanent
firewall-cmd --reload

echo "对${device}进行分区"

# 生成分区
{
  echo n # 新建分区
  echo p # 主分区
  echo 1 # 分区号
  echo   # 起始扇区（默认）
  echo   # 结束扇区（默认）

  echo w # 写入更改
} | fdisk $device


echo "分区完成。"

if [ ! -e "${device}1" ]; then
  error "${device}1磁盘分区失败"
  exit 1
fi

mkfs.ext4 ${device}1
rm -rf /data/minio_home/data/*
mount -t ext4 ${device}1 /data/minio_home/data/

if [ ! $? -eq 0 ]; then
  error "挂载失败: ${device}1 到 /data/minio_home/data/"
  exit 1
fi

device_uuid=$(blkid ${device}1 | awk -F '"' '{print $2}')

if grep -q "${device}1" /etc/fstab || grep -q "$device_uuid" /etc/fstab; then
  warn "${device}1 的配置已包含在 /etc/fstab 中，忽略配置。请自行检查/etc/fstab配置，是否有启动挂载Minio磁盘"
else
  echo "UUID=$device_uuid /data/minio_home/data                   ext4     defaults        0 0" >> /etc/fstab
fi

volumes=""

for node in "${nodes[@]}"; do
    # 将处理后的元素拼接到结果字符串中
    volumes+="http://$node/data/minio_home/data "
done

configContent='
MINIO_VOLUMES="'$volumes'"
MINIO_OPTS=" --address :19000 --console-address :19001 "

MINIO_ACCESS_KEY=admin
MINIO_SECRET_KEY=admin123455202221001'

echo "$configContent" > /usr/local/minio/minio.conf

systemctl start minio

if systemctl is-active --quiet "minio"; then
  echo "minio 服务已成功启动"
else
  warn "minio 服务启动失败，请查看日志并排查启动失败原因"
fi