nodes=()
vip=""
ip=$(ifconfig | awk 'NR==2{print $2}')

function success(){
  echo -e "\e[92m"$1"\e[0m"
}
function warn(){
  echo -e "\e[93m"$1"\e[0m"
}
function error(){
  echo -e "\e[91m"$1"\e[0m"
}

while getopts "n:v:" opt; do
  case ${opt} in
    n )
      nodes+=("$OPTARG")  # 将每个 -n 的值加入数组
      ;;
    v )
      vip=$OPTARG
      ;;
    \? )
      warn "Usage: cmd [-n node_ip] [-v vip]"
      exit 1
      ;;
  esac
done

node_length=${#nodes[@]}

if [[ $node_length -lt 2 ]]; then
  error "pgpool集群最少需要2个节点"
  exit 1
fi

if [[ -z "$vip" ]]; then
  error "VIP不允许为空"
  exit 1
fi

echo "进入目录$(dirname $0)"

cd "$(dirname "$0")"

SSH_OPTIONS="-o StrictHostKeyChecking=no -o UserKnownHostsFile=/dev/null -i ~/.ssh/id_rsa"

backend_account=""
backend_biz=""
backend_minio=""
backend_minio_console=""
backend_oo=""

for node in "${nodes[@]}"; do
backend_account+="  server $node:9881;
"
backend_biz+="  server $node:8881;
"
backend_minio+="  server $node:19000;
"
backend_minio_console+="  server $node:19001;
"
backend_oo+="  server $node:9002;
"
done

informat_next_conf_tmpl=$(cat './informat-next.conf.tmpl')

informat_next_conf=${informat_next_conf_tmpl//\{backend_account\}/$backend_account}
informat_next_conf=${informat_next_conf//\{backend_biz\}/$backend_biz}
informat_next_conf=${informat_next_conf//\{backend_minio\}/$backend_minio}
informat_next_conf=${informat_next_conf//\{backend_minio_console\}/$backend_minio_console}
informat_next_conf=${informat_next_conf//\{backend_oo\}/$backend_oo}
informat_next_conf=${informat_next_conf//\$/\\\$}

for node in "${nodes[@]}"; do

echo "更新${node}的informat-next.conf配置"

ssh -T $SSH_OPTIONS root@$node << EOF

enforce=$(getenforce)

if [[ \$enforce -eq 'Enforcing' ]]; then
  echo "关闭SELinux"
  setenforce 0
  sed -i "s/SELINUX=enforcing/SELINUX=disabled/" /etc/selinux/config
  echo -e "\e[92m关闭SELinux成功\e[0m"
fi

echo "添加80、443、9002端口策略"
firewall-cmd --zone=public --add-port=80/tcp --permanent
firewall-cmd --zone=public --add-port=9002/tcp --permanent
firewall-cmd --zone=public --add-port=443/tcp --permanent
firewall-cmd --reload

cp -vf /etc/nginx/conf.d/informat-next.conf /etc/nginx/conf.d/informat-next.conf.bak
echo "$informat_next_conf" > /etc/nginx/conf.d/informat-next.conf
systemctl restart nginx

if systemctl is-active --quiet "nginx"; then
  echo "nginx 服务已成功启动"
else
  echo -e "\e[91m服务启动失败，请查看日志并排查启动失败原因\e[0m"
fi

EOF

success "更新${node}配置成功"

((node_id++))
done

