nodes=()
vip=""
ip=$(ifconfig | awk 'NR==2{print $2}')

function success(){
  echo -e "\e[92m"$1"\e[0m"
}
function warn(){
  echo -e "\e[93m"$1"\e[0m"
}
function error(){
  echo -e "\e[91m"$1"\e[0m"
}

while getopts "n:v:" opt; do
  case ${opt} in
    n )
      nodes+=("$OPTARG")  # 将每个 -n 的值加入数组
      ;;
    v )
      vip=$OPTARG
      ;;
    \? )
      warn "Usage: cmd [-n node_ip] [-v vip]"
      exit 1
      ;;
  esac
done

node_length=${#nodes[@]}

if [[ $node_length -lt 2 ]]; then
  error "pgpool集群最少需要2个节点"
  exit 1
fi

if [[ -z "$vip" ]]; then
  error "VIP不允许为空"
  exit 1
fi

echo "进入目录$(dirname $0)"

cd "$(dirname "$0")"

SSH_OPTIONS="-o StrictHostKeyChecking=no -o UserKnownHostsFile=/dev/null -i ~/.ssh/id_rsa"

local_json_tmpl=$(cat './local.json.tmpl')

local_json=${local_json_tmpl//\{vip\}/$vip}

for node in "${nodes[@]}"; do

echo "更新${node}的local.json配置"

ssh -T $SSH_OPTIONS root@$node << EOF

cp -vf /etc/onlyoffice/documentserver/local.json /etc/onlyoffice/documentserver/local.json.bak
echo '$local_json' > /etc/onlyoffice/documentserver/local.json
systemctl restart supervisord

if systemctl is-active --quiet "supervisord"; then
  echo "onlyoffice 服务已成功启动"
else
  echo -e "\e[91m服务启动失败，请查看日志并排查启动失败原因\e[0m"
fi

EOF

success "更新${node}配置成功"

((node_id++))
done

