#!/bin/bash

nodes=()
vip=""
ip=$(ifconfig | awk 'NR==2{print $2}')

function success(){
  echo -e "\e[92m"$1"\e[0m"
}
function warn(){
  echo -e "\e[93m"$1"\e[0m"
}
function error(){
  echo -e "\e[91m"$1"\e[0m"
}

while getopts "n:v:" opt; do
  case ${opt} in
    n )
      nodes+=("$OPTARG")  # 将每个 -n 的值加入数组
      ;;
    v )
      vip=$OPTARG
      ;;
    \? )
      warn "Usage: cmd [-n node_ip] [-v vip]"
      exit 1
      ;;
  esac
done

node_length=${#nodes[@]}

if [[ $node_length -lt 2 ]]; then
  error "pgpool集群最少需要2个节点"
  exit 1
fi

if [[ -z "$vip" ]]; then
  error "VIP不允许为空"
  exit 1
fi

export PGPASSWORD="Informat-Next_#,.20221001"

SSH_OPTIONS="-o StrictHostKeyChecking=no -o UserKnownHostsFile=/dev/null -i ~/.ssh/id_rsa"

pghome=/data/pgsql/data

pgpassContent="$vip:9999:postgres:pgpool:$PGPASSWORD
"

pgpoolMd5Pass=$(pg_md5 -u=pgpool $PGPASSWORD)

backendSetting=""

wdSetting=""

heartbeatSetting=""

node_id=0



for node in "${nodes[@]}"; do

echo "在${node}执行命令"
ssh -T $SSH_OPTIONS root@$node << EOF

export PGPASSWORD="$PGPASSWORD"
echo '127.0.0.1:9898:pgpool:$PGPASSWORD' > /var/lib/pgsql/.pcppass
chown -R postgres:postgres /var/lib/pgsql/
chmod 600 /var/lib/pgsql/.pcppass
echo 'pgpool:$pgpoolMd5Pass' > /etc/pgpool-II/pcp.conf
echo "$node_id" > /etc/pgpool-II/pgpool_node_id

# 拷贝故障转移脚本
echo "拷贝故障转移脚本"
sed -i 's|/usr/pgsql-14|/usr/pgsql-13|g' /etc/pgpool-II/*.sample
sed -i 's|id_rsa_pgpool|id_rsa|g' /etc/pgpool-II/*.sample
sed -i 's|192.168.137.150|$vip|g' /etc/pgpool-II/*.sample
sed -i 's|REPLUSER=repl|REPLUSER=postgres|g' /etc/pgpool-II/*.sample

cp -p /etc/pgpool-II/failover.sh{.sample,}
cp -p /etc/pgpool-II/follow_primary.sh{.sample,}
cp -p /etc/pgpool-II/escalation.sh{.sample,}
cp -p /etc/pgpool-II/recovery_1st_stage{.sample,}
cp -p /etc/pgpool-II/pgpool_remote_start{.sample,}
chmod +x /etc/pgpool-II/{failover.sh,follow_primary.sh,escalation.sh,recovery_1st_stage,pgpool_remote_start}
sed -i 's|server1|${nodes[0]}|g' /etc/pgpool-II/escalation.sh
sed -i 's|server2|${nodes[1]}|g' /etc/pgpool-II/escalation.sh
sed -i 's|server3|${nodes[2]}|g' /etc/pgpool-II/escalation.sh

cp -vfp /etc/pgpool-II/recovery_1st_stage $pghome/

chown -R postgres:postgres /etc/pgpool-II
chown -R postgres:postgres $pghome

echo "拷贝完成"

echo "创建pgpool_recovery数据库插件"
psql -U postgres -d template1 -c "CREATE EXTENSION pgpool_recovery"
psql -U postgres -d postgres -c "CREATE EXTENSION pgpool_recovery"
echo "插件创建完成"

echo "向pgpool_passwd添加密码"
pg_md5 --md5auth --username=pgpool "$PGPASSWORD"
pg_md5 --md5auth --username=postgres "$PGPASSWORD"
echo "添加密码完成"

EOF

backendSetting+="backend_hostname$node_id = '$node'
backend_port$node_id = 5432
backend_weight$node_id = 1
backend_data_directory$node_id = '$pghome'
backend_flag$node_id = 'ALLOW_TO_FAILOVER'
backend_application_name$node_id = 'server$node_id'

"

wdSetting+="hostname$node_id = '$node'
wd_port$node_id = 9000
pgpool_port$node_id = 9999

"

heartbeatSetting+="heartbeat_hostname$node_id = '$node'
heartbeat_port$node_id = 9694
heartbeat_device$node_id = ''

"

pgpassContent+="$node:5432:postgres:postgres:$PGPASSWORD
"

((node_id++))

success "在${node}执行命令完成"
done

pgpoolConf="backend_clustering_mode = 'streaming_replication'
# - pgpool Connection Settings -
max_pool=100

listen_addresses = '*'
port=9999
socket_dir = '/var/run/postgresql'
pcp_listen_addresses = '*'
pcp_socket_dir = '/var/run/postgresql'

# - Streaming Replication Check
sr_check_user = 'pgpool'
sr_check_password = '$PGPASSWORD'

# - Health Check
health_check_period = 5
health_check_timeout = 30
health_check_user = 'pgpool'
health_check_password = '$PGPASSWORD'
health_check_max_retries = 3

search_primary_node_timeout = 10min

# - Backend Connection Settings -
$backendSetting

# - Failover configuration
failover_command = '/etc/pgpool-II/failover.sh %d %h %p %D %m %H %M %P %r %R %N %S'
follow_primary_command = '/etc/pgpool-II/follow_primary.sh %d %h %p %D %m %H %M %P %r %R'

# - Online Recovery Configurations
recovery_user = 'postgres'
recovery_password = 'Informat-Next_#,.20221001'
recovery_1st_stage_command = 'recovery_1st_stage'

# - Client Authentication Configuration
enable_pool_hba = on
pool_passwd = '/etc/pgpool-II/pool_passwd'

# - Watchdog Configuration
use_watchdog = on
delegate_IP = '$vip'

if_up_cmd = '/usr/bin/sudo /sbin/ip addr add \$_IP_\$/24 dev ens33 label ens33:1'
if_down_cmd = '/usr/bin/sudo /sbin/ip addr del \$_IP_\$/24 dev ens33'
arping_cmd = '/usr/bin/sudo /usr/sbin/arping -U \$_IP_\$ -w 1 -I ens33'

$wdSetting

wd_lifecheck_method = 'heartbeat'
wd_interval = 10

$heartbeatSetting

wd_heartbeat_keepalive = 2
wd_heartbeat_deadtime = 30

wd_escalation_command = '/etc/pgpool-II/escalation.sh'

failover_when_quorum_exists = on
failover_require_consensus = on
allow_multiple_failover_requests_from_node = on
enable_consensus_with_half_votes = on

# - Where to log -
log_destination = 'stderr'
logging_collector = on
log_directory = '/var/log/pgpool_log'
log_filename = 'pgpool-%Y-%m-%d_%H%M%S.log'
log_truncate_on_rotation = on
log_rotation_age = 1d
log_rotation_size = 10MB

pid_file_name = '/var/run/postgresql/pgpool.pid'

#------------------------------------------------------------------------------
# LOAD BALANCING MODE
#------------------------------------------------------------------------------
load_balance_mode = on"

# 修改配置并创建从节点
node_id=0
for node in "${nodes[@]}"; do
ssh -T $SSH_OPTIONS root@$node << EOF
export PGPASSWORD="$PGPASSWORD"
echo "$pgpassContent" > /var/lib/pgsql/.pgpass
chown -R postgres:postgres /var/lib/pgsql/
chmod 600 /var/lib/pgsql/.pgpass

cp -vf /etc/pgpool-II/pgpool.conf /etc/pgpool-II/pgpool.conf.bak
echo "$pgpoolConf" > /etc/pgpool-II/pgpool.conf

if [[ $node_id -ne 0 ]]; then
echo "创建备库"
cp -rf $pghome/ ${pghome}_bak/
rm -rf /data/pgsql/data/*
pg_basebackup -h ${nodes[0]} -p 5432 -U postgres -D $pghome/ -P -Xs -R
chown -R postgres:postgres $pghome/
chmod 700 $pghome/
echo "创建备库完成"
fi

systemctl restart postgresql-13.service
systemctl restart pgpool.service

EOF
if [[ $node_id -ge 1 ]]; then
  pcp_attach_node -U pgpool -n $node_id
else
  ip addr add $vip/24 dev ens33 label ens33:1
fi

((node_id++))
done

sleep 10
#查看集群节点状态
psql -d postgres -U pgpool -h 127.0.0.1 -p 9999 -c "show pool_nodes"