#!/bin/bash

port=$1
path=$2

function success(){
  echo -e "\e[92m"$1"\e[0m"
}
function warn(){
  echo -e "\e[93m"$1"\e[0m"
}
function error(){
  echo -e "\e[91m"$1"\e[0m"
}

usePort=$(netstat -nlp | grep :$port)

if [ ! -z "$usePort" ]; then
  error $port"端口已占用"
  exit 1
fi

if [ -z "$2" ]; then
  warn "集群路径不存在，使用默认集群地址"
  path='/usr/local/redis/redis-cluster'
fi

home=$path/redis_$port
mkdir -p $home/logs

ip=$(ifconfig | awk 'NR==2{print $2}')
if [ -z "$ip" ]; then
  warn "本机内网ip无法识别，请自行设置"
  read ip
fi

configContent="# 守护进行模式启动
daemonize yes

# 设置数据库数量，默认数据库为0
databases 16

# 绑定地址，需要修改
bind 0.0.0.0

# 绑定端口，需要修改
port $port
protected-mode no

# pid文件存储位置，文件名需要修改
pidfile redis.pid
appendonly yes

# log文件存储位置，文件名需要修改
logfile logs/redis.log

# RDB快照备份文件名，文件名需要修改
dbfilename redis.rdb

# 本地数据库存储目录，需要修改
dir $home

# 集群相关配置
# 是否以集群模式启动
cluster-enabled yes
cluster-announce-ip $ip

# 集群节点回应最长时间，超过该时间被认为下线
cluster-node-timeout 15000

# 生成的集群节点配置文件名，文件名需要修改
cluster-config-file nodes.conf

#masterauth redis123
#requirepass redis123"

echo "$configContent" > $home/redis.cnf
firewall-cmd --zone=public --add-port=$port/tcp --permanent
firewall-cmd --zone=public --add-port=1$port/tcp --permanent
firewall-cmd --reload
success "已创建于$home，命令启动：/usr/local/redis/bin/redis-server $home/redis.cnf"